#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__='pepekiko@gmail.com'

import json.scanner
from commoneasytools.resultOperation import ResultOperationEntity
from commoneasytools.resultType import ResultOperationType

class ResultData(object):
    """
        Complex model for returned data structured.
    """
    def __init__(self):
        self.__content = None
        self.__hasErrors = False
        self.__resultOperationEntities = []

    @property
    def Content(self):
        return self.__content

    @Content.setter
    def Content(self, value):
        self.__content = value

    @property
    def HasErrors(self):
        return self.__hasErrors
    
    @property
    def ResultOperationEntities(self):
        return self.__resultOperationEntities
    
    def AddResult(self, message: str, resultType: ResultOperationType):      
        """
            Method used for add an result to structure.
            message: field with message to add.
            resultType: one of class enums ResultOperationType            
        """
        try:   
            self.__resultOperationEntities.append(ResultOperationEntity(message,resultType))           
            if resultType == ResultOperationType.ERROR or resultType == ResultOperationType.EXCEPTION:
                self.__hasErrors = True
        except Exception as ex:
            raise ValueError('Exception %s in method business.ResultData.AddResult '%str(ex))
    
    def AddResultRange(self, resultRange: list):
        """
            Method used for add one Complex Result to list of results.
        """
        for res in resultRange:
            self.AddResult(res.Message,res.ResultType)
    
    def FormatResultOperationEntities(self):
        """
            Method used for return result structure formatting to text
        """
        message = ""
        for res in self.__resultOperationEntities:
            message = '%s %s - %s,'%(message,res.ResultType.name,res.Message)
        return message    

    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__='pepekiko@gmail.com'

import json
import inspect

class ObjectEncoder(json.JSONEncoder):
    """
        Class used for convert to json or to dict one object with properties.
        How to use:
            from common.jsonEncoder import ObjectEncoder as obEncoder
            json.dumps(result,cls=obEncoder,sort_keys=True)
        Note: Result is an object.
    """
    def default(self, obj):
        if hasattr(obj, "to_json"):
            return self.default(obj.to_json())
        elif hasattr(obj, "__dict__"):
            d = dict(
                (key, value)
                for key, value in inspect.getmembers(obj)
                if not key.startswith("__") and not key.startswith("_")
                and not inspect.isabstract(value)
                and not inspect.isbuiltin(value)
                and not inspect.isfunction(value)
                and not inspect.isgenerator(value)
                and not inspect.isgeneratorfunction(value)
                and not inspect.ismethod(value)
                and not inspect.ismethoddescriptor(value)
                and not inspect.isroutine(value)
            )
            return self.default(d)
        return obj


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__='pepekiko@gmail.com'

class ResultOperationEntity(object):
        """
            Secondary class used in ResultData. Provided sub structure with information about operation
        """
        def __init__(self, message, resultType):
            self.__message = message
            self.__resultType = resultType
        
        @property
        def Message(self):
            return self.__message

        @property
        def ResultType(self):
            return self.__resultType


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__='pepekiko@gmail.com'

from enum import Enum

class LoggerMinumLevel (Enum):
    """
        Minimun accepted logger level
    """
    CRITICAL=0
    DEBUG=1
    ERROR=2
    FATAL=3
    INFO=4
    WARNING=5


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__='pepekiko@gmail.com'

from enum import Enum

class ResultOperationType (Enum):
    """
        Enum with diferents types of result operations.
    """    
    ERROR = 0
    INFO = 1
    WARNING = 2
    SUCCESS = 3
    EXCEPTION = 4


#! usr/bin/python
# -*- coding: utf-8 *-* 
from distutils.core import setup
setup(
    name = 'commoneasytools',
    packages = ['commoneasytools'],
    version = '0.1.4',
    description = 'Common Easy Tools',
    author = 'José Fº Queija',
    author_email = 'pepekiko@gmail.com',
    url = 'https://github.com/jfqueija/commoneasytools',
    download_url = 'https://github.com/jfqueija/commontools/tarball/0.1.1',
    keywords = ['Common','Easy','Tools','Result','Model','Logger'],    
    classifiers = [  
        # Optional
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    install_requires=['python-logstash']
)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__='pepekiko@gmail.com'

import logging
import os
import sys
from datetime import datetime
from commoneasytools.loggerLevel import LoggerMinumLevel

class Log(object):
    """
        Class for merge logger options
    """ 
    def __init__(self):
        """
            Basic constructor
        """
        self.__logger = None
        

    def settingToFile(self,loggerPath:str, loggerFileName:str, minimunLevel: LoggerMinumLevel):
        """
            Method for setting logger configuration and save messages in file.
        """
        try:
            if not os.path.isdir(loggerPath):
                    os.mkdir(loggerPath)                        
            logging.basicConfig(filename='%s/%s_%s.log'%(loggerPath,loggerFileName, datetime.today().strftime('%Y%m%d')),        
                filemode='a',format='%(asctime)s - [%(levelname)s] -  %(message)s',
                datefmt='%Y/%m/%d %H:%M:%S',
                level=self.__getlevel(minimunLevel.value))                       
            logging.getLogger('python')
            logger = logging.getLogger('python')
            logger.setLevel(self.__getlevel(minimunLevel.value))   
            self.__logger = logger
        except Exception:
            self.__logger = None            
        return self.__logger

    def settingToLogstash(self, host:str, port:int, minimunLevel: LoggerMinumLevel):
        """
            Method for setting logger configuration using logstash.
            @host: Ip or Name of Logstash server.
            @port: Logstash Listen Port. Normally is 5959. Review you logstash configuration.
            We can add extra information when send message using dictionaries. Sample:
                extraFields = {'Field1': value1, 'Field2': value2}
                logger.info(msg='Message',extra=extraFields)
        """
        import logstash
        try:                                             
            logger = logging.getLogger('python-logstash')
            logger.setLevel(self.__getlevel(minimunLevel.value))  
            logger.addHandler(logstash.LogstashHandler(host, port, version=1))                         
            self.__logger = logger
        except Exception:
            self.__logger = None
        return self.__logger

    def __getlevel(self,level):
        """
            Private method for select level request by parameter
        """
        levelSelected = None
        if level == LoggerMinumLevel.CRITICAL.value:
            levelSelected = logging.CRITICAL
        elif level == LoggerMinumLevel.DEBUG.value:
            levelSelected = logging.DEBUG
        elif level == LoggerMinumLevel.ERROR.value:
            levelSelected = logging.ERROR
        elif level == LoggerMinumLevel.FATAL.value:
            levelSelected = logging.FATAL
        elif level == LoggerMinumLevel.INFO.value:
            levelSelected = logging.INFO
        elif level == LoggerMinumLevel.WARNING.value:
            levelSelected = logging.WARNING
        else:
            levelSelected = logging.DEBUG
        return levelSelected

